/** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* BiDiPro/AVR v0.1.0                 avr.c -- functions for working w/ AVRs
* Copyright (c) Oleg O. Chukaev, 2007, 2008  <oleg.chukaev@mail.ru>
* Based on AVRPP - AVR Parallel Programming Controller  R0.39 (C)ChaN, 2007
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/


/*-------------------------------------------------------------------------
  ᮪ 
---------------------------------------------------------------------------

2007-07-01	0leg	-
2007-07-30	0leg	প ISP


---------------------------------------------------------------------------
  ᮪ ⫨稩  㬥樨  ᮡ ॠ樨
---------------------------------------------------------------------------

1. HVS: ⥭ Flash  EEPROM  ATtiny13/24/44/84/25/45/85. 砫
   㦠 訩  , ⮬ 訩 ,  .. .
2. HVS:  ࠭ Flash  ATtiny24/44/84. ய饭 
   樨.
3. HVS:  ࠭ EEPROM   . 訩   㦠
   _᫥_ 㧪    ࠭,   -- __६_.
4. PAR/HVS:  C_RD_PRG 믮 ⮫쪮  ⥭ 0- 
   Flash; 訥    ⮫쪮 ᫥ ⥭ ᮮ. . ⮢.
   .. Flash    ந쭮    ந쭮
   浪.  C_RD_EEP 믮 ⮫쪮  ⥭ 0- 
   EEPROM. ..  EEPROM    ந쭮 .  
    _१_ 㢥稢 ᪮ ⥭...


---------------------------------------------------------------------------
   ணࠬ஢
---------------------------------------------------------------------------

**** ࠫ ०. ⥭. ****

⥭ Flash
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x02 (C_RD_PRG)
2. (XA1, XA0, BS2, BS1) <- (0, 0, 1, 0);  D7..D0 <- .  
     ᫥ 믮 ⮫쪮  室    
   65536 ᫮.
3. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 1);  D7..D0 <- .  
     ᫥ 믮 ⮫쪮  室    
   256 ᫮.
4. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0);  D7..D0 <- .  
5. BS1 <- 0; ⠥ . 
6. BS1 <- 1; ⠥ . 

⥭ EEPROM
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x03 (C_RD_EEP)
2. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 1);  D7..D0 <- .  
     ᫥ 믮 ⮫쪮  室    
   256 .
3. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0);  D7..D0 <- .  
4. BS1 <- 0; ⠥ 

⥭ 䨪
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x08 (C_RD_SIG)
2. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0);  D7..D0 <-  (0..2)
3. BS1 <- 0; ⠥ 

⥭ ஢ ⮢
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x08 (C_RD_SIG)
2. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0);  D7..D0 <-  (0..3)
3. BS1 <- 1; ⠥ 

⥭ Fuse-  Lock-⮢
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x04 (C_RD_FB)
2.  tiny/mega:
     Low:  (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0)
     High: (XA1, XA0, BS2, BS1) <- (1, 0, 1, 1)
     Ext:  (XA1, XA0, BS2, BS1) <- (1, 0, 1, 0)
     Lock: (XA1, XA0, BS2, BS1) <- (0, 0, 0, 1)
    s1200, s2313:
     Low+Lock: (XA1, XA0, BS2, BS1) <- (0, 0, 0, 1)
3. ⠥ 
   XA_1  ⥭ High-  Ext-fuse 㦥  ATtiny26  ATtiny2313, ..
   뢮 XA1 ⠪ 믮  㭪樨 뢮 BS2   .


****  HVS. ⥭. ****

⥭ Flash
1. SII: 0x4c (I_LDCMD); SDI: 0x02 (C_RD_PRG)
2. SII: 0x0c (I_LDAL);  SDI: .  
3. SII: 0x1c (I_LDAH);  SDI: .  
     ᫥ 믮 ⮫쪮  室    
   256 ᫮.
4. SII: 0x68 (I_RDLL1); SDI: 0x00
5. SII: 0x6c (I_RDLL2); SDI: 0x00
   ⠥ 訩 .
6. SII: 0x78 (I_RDLH1); SDI: 0x00
7. SII: 0x7c (I_RDLH2); SDI: 0x00
   ⠥ 訩 .

⥭ EEPROM
1. SII: 0x4c (I_LDCMD); SDI: 0x03 (C_RD_EEP)
2. SII: 0x0c (I_LDAL);  SDI: .  
3. SII: 0x1c (I_LDAH);  SDI: .  
     ᫥ 믮 ⮫쪮  室   256-⮢
   .
4. SII: 0x68 (I_RDLL1); SDI: 0x00
5. SII: 0x6c (I_RDLL2); SDI: 0x00
   ⠥ .

⥭ 䨪
1. SII: 0x4c (I_LDCMD); SDI: 0x08 (C_RD_SIG)
2. SII: 0x0c (I_LDAL);  SDI:   (0..2)
3. SII: 0x68 (I_RDLL1); SDI: 0x00
4. SII: 0x6c (I_RDLL2); SDI: 0x00
   ⠥ .

⥭ ஢ ⮢
1. SII: 0x4c (I_LDCMD); SDI: 0x08 (C_RD_SIG)
2. SII: 0x0c (I_LDAL);  SDI:   (0..3)
3. SII: 0x78 (I_RDLH1); SDI: 0x00
4. SII: 0x7c (I_RDLH2); SDI: 0x00
   ⠥ .

⥭ Lock-  Fuse-⮢
1. SII: 0x4c (I_LDCMD); SDI: 0x04 (C_RD_FB)
2. Low:  SII: 0x68 (I_RDLL1);  SDI: 0x00
   High: SII: 0x7a (I_RDHH1);  SDI: 0x00
   Ext:  SII: 0x6a (I_RDHL1);  SDI: 0x00
   Lock: SII: 0x78 (I_RDLH1);  SDI: 0x00
3. Low:  SII: 0x6c (I_RDLL2);  SDI: 0x00
   High: SII: 0x7e (I_RDHH2);  SDI: 0x00
   Ext:  SII: 0x6e (I_RDHL2);  SDI: 0x00
   Lock: SII: 0x7c (I_RDLH2);  SDI: 0x00
   ⠥ .


****  ISP. ⥭. ****

⥭ Flash
1. 0x28 (ISP_C_RD_PRGH)  ⥭ .  
   0x20 (ISP_C_RD_PRGL)  ⥭ . 
2. .  
3. .  
4.  
   ࠧ஬ flash >= 256K । 蠣 1 믮:
a) 0x4d (ISP_C_LD_ADRX)
b) 0x00
c) .  
d) 0x00

⥭ EEPROM
1. 0xa0 (ISP_C_RD_EEP)
2. .  
3. .  
4.  

⥭ 䨪
1. 0x30 (ISP_C_RD_SIG)
2. 0x00
3.  
4.  

⥭ ஢ ⮢
1. 0x38 (ISP_C_RD_CAL)
2. 0x00
3.  
4.  

⥭ Lock-  Fuse-⮢
I.   s1200, s2313, s4414, s8515:  0 -- fuse/lock- 㯭  ISP,
     頥 祭  㬮砭.
II.  s2323, s2343, s4434, s8535, tn22:  1 -- 1 fuse-, ᮢ 
     lock-⮬.  7  6 -- ᮮ. LB1  LB2.
     1. 0x58 (ISP_C_RD_FLB1)
     2. *
     3. *
     4.  .
     !!! ஢  tiny22 !!!
     Lock- 頥  ⮬ ,  室   
     (.... .21.).
III. Tiny/Mega
     Lock:
       1. 0x58 (ISP_C_RD_FLB1)
       2. 0x00
     Low:
       1. 0x50 (ISP_C_RD_FB1)
       2. 0x00
     High:
       1. 0x58 (ISP_C_RD_FLB1)
       2. 0x08
     Ext:
       1. 0x50 (ISP_C_RD_FB1)
       2. 0x08
     3. 0x00
     4.  


**** ࠫ ०. . ****

 ࠭ Flash
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x10 (C_WR_PRG)
2. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0);  D7..D0 <- .  
3. (XA1, XA0, BS2, BS1) <- (0, 1, x, 0);  D7..D0 <- .  
4. (XA1, XA0, BS2, BS1) <- (0, 1, x, 1);  D7..D0 <- .  
5. BS1 <- 1, PAGEL <- _/~\_
6.  2..5   ࠭
7. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 1);  D7..D0 <- .  
8. (XA1, XA0, BS2, BS1) <- (0, 0, 1, 0);  D7..D0 <- .  
9. (XA1, XA0, BS2, BS1) <- (x, x, 0, 0);  -WR <- ~\_/~,  1  RDY/-BSY

 ࠭ EEPROM
1. (XA1, XA0, BS2, BS1) <- (1, 0, x, 0);  D7..D0 <- 0x11 (C_WR_EEP)
2. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 1);  D7..D0 <- .  
3. (XA1, XA0, BS2, BS1) <- (0, 0, 0, 0);  D7..D0 <- .  
4. (XA1, XA0, BS2, BS1) <- (0, 1, x, 0);  D7..D0 <-  
5. BS1 <- 0, PAGEL <- _/~\_
6.  3..5   ࠭
7. (XA1, XA0, BS2, BS1) <- (x, x, 0, 0);  -WR <- ~\_/~,  1  RDY/-BSY


****  HVS. . ****

 ࠭ Flash
1. SII: 0x4c (I_LDCMD); SDI: 0x10 (C_WR_PRG)
2. SII: 0x0c (I_LDAL);  SDI: .  
3. SII: 0x2c (I_LDDL);  SDI: .  
4.  ATtiny24x/44x/84x:
   SII: 0x6d (I_PSTL1); SDI: 0x00
   SII: 0x6c (I_PSTL2); SDI: 0x00
5. SII: 0x3c (I_LDDH);  SDI: .  
6. SII: 0x7d (I_PSTH1); SDI: 0x00
7. SII: 0x7c (I_PSTH2); SDI: 0x00
8.  2..7   ࠭.
9. SII: 0x1c (I_LDAH);  SDI: .  
     ᫥ 믮 ⮫쪮  室   256-᫮
   .
A. SII: 0x64 (I_WRLL1); SDI: 0x00
B. SII: 0x6c (I_WRLL2); SDI: 0x00
C.  1  SDO.

 ࠭ EEPROM
1. SII: 0x4c (I_LDCMD); SDI: 0x11 (C_WR_EEP)
2. SII: 0x0c (I_LDAL);  SDI: .  
3. SII: 0x1c (I_LDAH);  SDI: .  
4. SII: 0x2c (I_LDDL);  SDI:  
5. SII: 0x6d (I_PSTL1); SDI: 0x00
6. SII: 0x6c (I_PSTL2); SDI: 0x00
7.  2..6   ࠭.
8. SII: 0x64 (I_WRLL1); SDI: 0x00
9. SII: 0x6c (I_WRLL2); SDI: 0x00
A.  1  SDO.


**** ࠫ ०.  fuse/lock. ****

 Lock-
1. (XA1, XA0) <- (1, 0); D7..0 <- 0x20 (C_WR_LB)
2. (XA1, XA0) <- (0, 1); D7..0 <- 祭 
3. (BS2, BS1) <- (0, 0)
4.   .
5.  1  RDY/-BSY.

 Fuse-⮢
1. (XA1, XA0) <- (1, 0); D7..0 <- 0x40 (C_WR_FB)
2. (XA1, XA0) <- (0, 1); D7..0 <- 祭 
3. Low:  (BS2, BS1) <- (0, 0)
   High: (BS2, BS1) <- (0, 1)
   Ext:  (BS2, BS1) <- (1, 0)
4.   .
5.  tiny/mega:
      1  RDY/-BSY.
    s1200:
      fuse_wait .
6. (BS2, BS1) <- (0, 0)


****  HVS.  fuse/lock. ****

 Lock-
1. SII: 0x4c (I_LDCMD); SDI: 0x20 (C_WR_LB)
2. SII: 0x2c (I_LDDL);  SDI: lock-
3. SII: 0x64 (I_WRLL1); SDI: 0x00
4. SII: 0x6c (I_WRLL2); SDI: 0x00
5.  1  SDO.

 Fuse-⮢  ० HVS.
1. SII: 0x4c (I_LDCMD); SDI: 0x40 (C_WR_FB)
2. SII: 0x2c (I_LDDL);  SDI: fuse-
3. Low:  SII: 0x64 (I_WRLL1); SDI: 0x00
   High: SII: 0x74 (I_WRLH1); SDI: 0x00
   Ext:  SII: 0x66 (I_WRHL1); SDI: 0x00
4.  ATtiny11  2.5 .
5. Low:  SII: 0x6c (I_WRLL2); SDI: 0x00
   High: SII: 0x7c (I_WRLH2); SDI: 0x00
   Ext:  SII: 0x6e (I_WRHL2); SDI: 0x00
6.  1  SDO ( , ஬ ATtiny11).


****  ISP.  fuse/lock. ****

 Lock-  Fuse-⮢.
I.  ᥬ Mega  Tiny.
   1. 0xac (ISP_C_WR_FLB)
   2. 0xa0 (ISP_C_WR_FLBL) -- . Fuse-
      0xa8 (ISP_C_WR_FLBH) -- . Fuse-
      0xa4 (ISP_C_WR_FLBX) -- . Fuse-
      0xe0 (ISP_C_WR_FLBK) -- Lock-
   3. 0x00
   4.  (Fuse-  Lock-)


**** ࠫ ०. ࠭. ****

1. (XA1, XA0) <- (1, 0); D7..0 <- 0x80 (C_ERASE)
2.   .
3.  1  RDY/-BSY.


****  HVS. ࠭. ****

1. SII: 0x4c (I_LDCMD); SDI: 0x80 (C_ERASE)
2. SII: 0x64 (I_WRLL1); SDI: 0x00
3. SII: 0x6c (I_WRLL2); SDI: 0x00
4. ...
5.  ATtiny11/12/15/s2323/s2343:
   SII: 0x4c (I_LDCMD); SDI: 0x00 (C_NOP)
6.  1  SDO.


****  HVS. ࠭. ****

1. 0xac (ISP_C_ERASE1)
2. 0x80 (ISP_C_ERASE2) ( ⮫쪮  7..5)
3. 0x00 \  
4. 0x00 / 묨
5.  40 .
6. ATtiny12  .  ४⭮ ࠭ ᫥ 뫪 㪠
      ࠧ   ० ணࠬ஢,   0 
   Flash'  0xff   ࠧ  ⠫ (蠣 1..5).
7.  ࠧ   ० ணࠬ஢.

-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------
  砥 䠩
-------------------------------------------------------------------------*/

#include <string.h>		/*  strcmp() */
#include "debug.h"		/* 뢮 ⫠筮 ଠ樨 */
#include "avr.h"		/*   ࠡ  AVR  . */
#include "cmds.h"		/*   ࠡ  AVR  . */
#include "hwctrl.h"		/* ஢ -樨 */


/*-------------------------------------------------------------------------
  譨 ६
-------------------------------------------------------------------------*/

extern const DEVPROP *device;	/* Pointer to the current device property */


/*-------------------------------------------------------------------------
   㭪樨
-------------------------------------------------------------------------*/

/****************************************************************************
* ⥭ .
* ࠬ:
*	src -- 㤠 : FLASH/EEPROM/SIGNATURE/CALIBS/FUSE.
*	adr --  .
* 頥:
*	⠭ .
****************************************************************************/

byte read_byte (int src, dword adr) {

	byte s, cmd, r1, r2;

	switch (port_mode) {
	case PAR:
		switch (src) {
		case FLASH:
			if (adr & 1)
				return (rcv_byte (BS_1));
			adr >>= 1;
			if (adr == 0)
				set_byte (XA_1, C_RD_PRG);
			if (((adr & 0xffff) == 0) && (device->flash_size > (128*1024L)))
				set_byte (BS_2, (byte)(adr >> 16));
			if ((adr & 0xff) == 0)
				set_byte (BS_1, (byte)(adr >> 8));
			set_byte (0, (byte)adr);
			return (rcv_byte (0));

		case EEPROM:
			if (adr == 0)
				set_byte (XA_1, C_RD_EEP);
			if ((adr & 0xff) == 0)
				set_byte (BS_1, (byte)(adr >> 8));
			set_byte (0, (byte)adr);
			return (rcv_byte (0));

		case SIGNATURE:
			set_byte (XA_1, C_RD_SIG);
			set_byte (0, (byte)adr);
			return (rcv_byte (0));

		case CALIBS:
			set_byte (XA_1, C_RD_SIG);
			set_byte (0, (byte)adr);
			return (rcv_byte (BS_1));

		case FUSE:
			set_byte (XA_1, C_RD_FB);
			switch (adr) {
			case F_LOW:
				/* !!! ஢  AT90S* !!! */
				s = device->fuses ? 0 : BS_1;
				break;
			case F_HIGH:
				s = XA_1 | BS_2 | BS_1;
				break;
			case F_EXTEND:
				s = XA_1 | BS_2;
				break;
			case F_LOCK:
				s = BS_1;
				break;
			default:
				return (0xff);
			}
			return (rcv_byte (s));

		default:
			return (0xff);
		} /* switch (src) */

	case HVS:
	case HVS15:
		switch (src) {
		case FLASH:
			if (adr == 0)
				xfer8 (I_LDCMD, C_RD_PRG);
			if (adr & 1) {
				xfer8 (I_RDLH1, 0);
				return (xfer8 (I_RDLH2, 0));
			}
			/* !!!  2  3 室  ⠬? */
			if ((adr & 0x1ff) == 0)
				xfer8 (I_LDAH, (byte)(adr >> 9));
			xfer8 (I_LDAL, (byte)(adr >> 1));
			xfer8 (I_RDLL1, 0);
			return (xfer8 (I_RDLL2, 0));

		case EEPROM:
			if (adr == 0)
				xfer8 (I_LDCMD, C_RD_EEP);
			/* !!!  2  3 室  ⠬? */
			if ((adr & 0xff) == 0)
				xfer8 (I_LDAH, (byte)(adr >> 8));
			xfer8 (I_LDAL, (byte)adr);
			xfer8 (I_RDLL1, 0);
			return (xfer8 (I_RDLL2, 0));

		case SIGNATURE:
			xfer8 (I_LDCMD, C_RD_SIG);
			xfer8 (I_LDAL, (byte)adr);
			xfer8 (I_RDLL1, 0);
			return (xfer8 (I_RDLL2, 0));

		case CALIBS:
			xfer8 (I_LDCMD, C_RD_SIG);
			xfer8 (I_LDAL, (byte)adr);
			xfer8 (I_RDLH1, 0);
			return (xfer8 (I_RDLH2, 0));

		case FUSE:
			xfer8 (I_LDCMD, C_RD_FB);
			switch (adr) {
			case F_LOW:
				xfer8 (I_RDLL1, 0);
				return (xfer8 (I_RDLL2, 0));
			case F_HIGH:
				xfer8 (I_RDHH1, 0);
				return (xfer8 (I_RDHH2, 0));
			case F_EXTEND:
				xfer8 (I_RDHL1, 0);
				return (xfer8 (I_RDHL2, 0));
			case F_LOCK:
				xfer8 (I_RDLH1, 0);
				return (xfer8 (I_RDLH2, 0));
			default:
				return (0xff);
			}
		default:
			return (0xff);
		} /* switch (src) */

	case ISP:
		switch (src) {
		case FLASH :
			cmd = (byte)((adr & 1) ? ISP_C_RD_PRGH : ISP_C_RD_PRGL);
			adr >>= 1;
			if ((device->flash_size > (128*1024L))
				&& ((adr & 0xffff) == 0)) {	/* Load extended address if needed */
				spi_xmit (ISP_C_LD_ADRX);	/* extended address command */
				spi_xmit (0);			/* 0 */
				spi_xmit ((byte)(adr >> 16));	/* address extended */
				spi_xmit (0);			/* 0 */
			}
			break;

		case EEPROM :
			cmd = ISP_C_RD_EEP;
			break;

		case SIGNATURE :
			cmd = ISP_C_RD_SIG;
			break;

		case CALIBS:
			cmd = ISP_C_RD_CAL;
			break;

		case FUSE:
			if (device->fuse_type == 0)
				return (device->fuse_default[0]);

			if (device->fuse_type == 1) {
				spi_xmit (ISP_C_RD_FLB1);
				spi_xmit (0);
				spi_xmit (0);
				s = spi_rcvr ();
				if (adr == F_LOCK) {
					r1 = (s >> 6) | 0xfd;
					r2 = (s >> 4) | 0xfb;
					return (r1 & r2);
				} else {
					return (s | ~device->fuse_mask[0]);
				}
			}

			switch (adr) {
			case F_LOW:			/* 2..6 */
				spi_xmit (ISP_C_RD_FB1);
				spi_xmit (0);
				break;
			case F_HIGH:			/* 5..6 */
				spi_xmit (ISP_C_RD_FLB1);
				spi_xmit (8);
				break;
			case F_EXTEND:			/* 6 */
				spi_xmit (ISP_C_RD_FB1);
				spi_xmit (8);
				break;
			case F_LOCK:
				spi_xmit (ISP_C_RD_FLB1);
				spi_xmit (0);
				break;
			default:
				return (0xff);
			}
			spi_xmit (0);
			return (spi_rcvr ());

		default:
			return (0xff);
		}
		spi_xmit (cmd);				/* Read command */
		spi_xmit ((byte)(adr >> 8));		/* Address high */
		spi_xmit ((byte)adr);			/* Address low */
		return (spi_rcvr ());			/* Receive data */

	default:
		return (0xff);
	}
}


/****************************************************************************
*  .
* ࠬ:
*	dst -- 㤠 뢠: FLASH/EEPROM/FLASH_NS.
*		FLASH_NS ⮫쪮  ISP,  뢠   砥,
*		 ᫨  ࠢ 0xff.
*	adr --  .
*	wd  -- , 騩 .
* 頥:
*	0 --  訫 ᯥ譮.
*	1 -- 訡, . ⠩.
****************************************************************************/

int write_byte (int dst, dword adr, byte wd) {

	int n;

	switch (port_mode) {
	case PAR:				/* Parallel Mode */
		switch (dst) {
		case FLASH:
			if ((adr & 1) == 0) {
				if (adr == 0)
					set_byte (XA_1, C_WR_PRG);
				if ((adr & 0x1FF) == 0)
					set_byte (BS_1, (byte)(adr >> 9));
				set_byte (0, (byte)(adr >> 1));
			}
			if (wd == 0xFF)
				return (0);	/* Skip if the value is 0xFF */
			set_byte (XA_0, wd);
			stb_wr ((byte)(adr & 1 ? BS_1 : 0), 0);
			break;

		case EEPROM:
			if (adr == 0)
				set_byte (XA_1, C_WR_EEP);
			if ((adr & 0xFF) == 0)
				set_byte (BS_1, (byte)(adr >> 8));
			set_byte (0, (byte)adr);
			set_byte (XA_0, wd);
			stb_wr (0, 0);
			break;

		default:
			return (1);
		} /* switch (dst) */
		break;

	case HVS:			/* HVS mode */
	case HVS15:			/* HVS15 mode */
		switch (dst) {
		case FLASH:
			if (adr == 0)
				xfer8 (I_LDCMD, C_WR_PRG);
			if ((adr & 1) == 0) {
				if ((adr & 0x1FF) == 0)
					xfer8 (I_LDAH, (byte)(adr >> 9));
				xfer8 (I_LDAL, (byte)(adr >> 1));
				if (wd == 0xFF)
					return (0);	/* Skip if the value is 0xFF */
				xfer8 (I_LDDL, wd);
				xfer8 (I_WRLL1, 0);
				xfer8 (I_WRLL2, 0);
			} else {
				if (wd == 0xFF)
					return (0);	/* Skip if the value is 0xFF */
				xfer8 (I_LDDH, wd);
				xfer8 (I_WRLH1, 0);
				xfer8 (I_WRLH2, 0);
			}
			break;

		case EEPROM:
			if (adr == 0)
				xfer8 (I_LDCMD, C_WR_EEP);
			if ((adr & 0xFF) == 0)
				xfer8 (I_LDAH, (byte)(adr >> 8));
			xfer8 (I_LDAL, (byte)adr);
			xfer8 (I_LDDL, wd);
			xfer8 (I_WRLL1, 0);
			xfer8 (I_WRLL2, 0);
			break;

		default:
			return (1);
		} /* switch (dst) */
		break;

	case ISP:			/* ISP mode */
		switch (dst) {
		case FLASH:
			/* Skip if the value is 0xFF */
			if (wd == 0xff)
				return (0);
			/* FALLTHROUGH */

		case FLASH_NS:
			spi_xmit ((byte)((adr & 1) ? ISP_C_WR_PRGH : ISP_C_WR_PRGL));
			spi_xmit ((byte)(adr >> 9));
			spi_xmit ((byte)(adr >> 1));
			spi_xmit (wd);
			if ((device->poll_data == wd)		/* Write data is equal to poll data */
				|| (strcmp (device->name, "90S1200") == 0)) /* 90S1200 cannot be polled */
			{
				delay_ms (device->flash_wait);
				return (0);
			}
			break;

		case EEPROM :
			spi_xmit (ISP_C_WR_EEP);
			spi_xmit ((byte)(adr >> 8));
			spi_xmit ((byte)adr);
			spi_xmit (wd);
			if ((device->poll_data == wd)
				|| (device->poll_data == (byte)~wd)
				|| (strcmp (device->name, "90S1200") == 0)) /* 90S1200 cannot be polled */
			{
				delay_ms (device->eeprom_wait);
				return (0);
			}
			break;

		default:
			return (1);
		} /* switch (dst) */
		/* Wait for end of programming (Polling) */
		for (n = 0; n < 200; n++) {
			if (read_byte (dst, adr) == wd) {
				return (0);
			}
		}
		return (1);	/* Polling time out */

	default:
		return (1);
	} /* switch (port_mode) */
	return (wait_ready ());	/* Wait for end of internal process */
}


/****************************************************************************
*  ࠭.
* ࠬ:
*	dst -- 㤠 뢠: FLASH/EEPROM.
*		 ࠭ EEPROM  ० ISP ,  ⮩
*		㭪樥   ন, ..  ⮬ ०
*		  ⮢  EEPROM ___ ⨯ .
*	adr --  .    ࠭ ࠭.
*	wd  -- 㪠⥫   ࠭, 饩 .
* 頥:
*	0 --  訫 ᯥ譮.
*	1 -- 訡, . ⠩.
****************************************************************************/

int write_page (int dst, dword adr, const byte *wd) {

	byte d = 0xff;
	int n;

	/* Skip page if all data in the Flash page are 0xFF */
	if (dst == FLASH) {
		for (n = 0; n < device->flash_page; n++)
			d &= wd[n];
		if (d == 0xff)
			return (0);
	}

	switch (port_mode) {
	case PAR:
		switch (dst) {
		case FLASH:
			set_byte (XA_1, C_WR_PRG);
			for (n = 0; n < device->flash_page; n += 2) {
				set_byte (0, (byte)((adr + n) >> 1));
				set_byte (XA_0, wd[n]);
				set_byte (XA_0 | BS_1, wd[n+1]);
				stb_pagel ();
			}
			if (device->flash_size > 128*1024L)
				set_byte (BS_2, (byte)(adr >> 17));
			set_byte (BS_1, (byte)(adr >> 9));
			stb_wr (0, 0);
			break;

		case EEPROM:
			set_byte (XA_1, C_WR_EEP);
			for (n = 0; n < device->eeprom_page; n++) {
				set_byte (0, (byte)(adr + n));
				set_byte (XA_0, wd[n]);
				stb_pagel ();
			}
			set_byte (BS_1, (byte)(adr >> 8));
			stb_wr (0, 0);
			break;

		default:
			return (1);
		}
		/* Wait for end of internal process */
		return (wait_ready ());

	case HVS:
		switch (dst) {
		case FLASH:
			xfer8 (I_LDCMD, C_WR_PRG);
			for (n = 0; n < device->flash_page; n += 2) {
				xfer8 (I_LDAL, (byte)((adr + n) >> 1));
				xfer8 (I_LDDL, wd[n]);
				/* !!! . . 4  ATtiny24x/44x/84x !!! */
				xfer8 (I_LDDH, wd[n + 1]);
				xfer8 (I_PSTH1, 0);
				xfer8 (I_PSTH2, 0);
			}
			xfer8 (I_LDAH, (byte)(adr >> 9));
			xfer8 (I_WRLL1, 0);
			xfer8 (I_WRLL2, 0);
			break;

		case EEPROM:
			xfer8 (I_LDCMD, C_WR_EEP);
			for (n = 0; n < device->eeprom_page; n++) {
				xfer8 (I_LDAL, (byte)(adr + n));
				xfer8 (I_LDDL, wd[n]);
				xfer8 (I_PSTL1, 0);
				xfer8 (I_PSTL2, 0);
			}
			/* !!!   믮  横 */
			/* 㧪  ࠭ !!! */
			xfer8 (I_LDAH, (byte)(adr >> 8));

			xfer8 (I_WRLL1, 0);
			xfer8 (I_WRLL2, 0);
			break;

		default:
			return (1);
		}
		/* Wait for end of internal process */
		return (wait_ready ());

	case HVS15:
		/* ATtiny15  ন ࠭  */
		return (1);

	case ISP:
		switch (dst) {
		case FLASH:
			/* Load the page data into page buffer */
			for (n = 0; n < device->flash_page; n++) {
				spi_xmit ((byte)((n & 1) ? ISP_C_WR_PRGH : ISP_C_WR_PRGL));
				spi_xmit ((byte)((adr + n) >> 9));
				spi_xmit ((byte)((adr + n) >> 1));
				spi_xmit (wd[n]);
			}

			/* Load extended address if needed */
			if (device->flash_size > 128*1024L) {
				spi_xmit (ISP_C_LD_ADRX);
				spi_xmit (0);
				spi_xmit ((byte)(adr >> 17));
				spi_xmit (0);
			}

			/* Start page programming */
			spi_xmit (ISP_C_WR_PAGE);
			spi_xmit ((byte)(adr >> 9));
			spi_xmit ((byte)(adr >> 1));
			spi_xmit (0);

			/* Wait for page write time */
			delay_ms (device->flash_wait);
			return (0);

		case EEPROM:
			return (1);

		default:
			return (1);
		}

	default:
		return (1);
	}
}


/****************************************************************************
*  Fuse-  Lock-⮢.
* ࠬ:
*	dst -- 㤠 뢠 : F_LOW/F_HIGH/F_EXTEND/F_LOCK.
*	val -- 祭 .
* 頥:
*	0 --  訫 ᯥ譮.
*	1 -- 訡, . ⠩.
****************************************************************************/

int write_fuselock (int dst, byte val) {

	switch (port_mode) {
	case PAR:
		if (dst == F_LOCK) {
			set_byte (XA_1, C_WR_LB);
			set_byte (XA_0, val);
			stb_wr (0, 0);
		} else {
			set_byte (XA_1, C_WR_FB);
			set_byte (XA_0, val);
			switch (dst) {
			case F_LOW:
				stb_wr (0, device->fuse_wait);
				break;
			case F_HIGH:
				stb_wr (BS_1, device->fuse_wait);
				break;
			case F_EXTEND:
				/* XA_1  㦥  ATtiny2313, */
				/* .. 뢮 XA1 ⠪ 믮  */
				/* 㭪樨 뢮 BS2 */
				stb_wr (XA_1 | BS_2, device->fuse_wait);
				break;
			}
		}
		/* Wait for end of internal process */
		return (wait_ready ());

	case HVS:
	case HVS15:
		if (dst == F_LOCK) {	/* Device Lock byte */
			xfer8 (I_LDCMD, C_WR_LB);
			xfer8 (I_LDDL, val);
			xfer8 (I_WRLL1, 0);
			xfer8 (I_WRLL2, 0);
		} else {
			xfer8 (I_LDCMD, C_WR_FB);
			xfer8 (I_LDDL, val);
			switch (dst) {
			case F_LOW:	/* Fuse Low byte */
				xfer8 (I_WRLL1, 0);
				delay_ms (device->fuse_wait);
				xfer8 (I_WRLL2, 0);
				break;
			case F_HIGH:	/* Fuse High byte */
				xfer8 (I_WRLH1, 0);
				xfer8 (I_WRLH2, 0);
				break;
			case F_EXTEND:	/* Fuse Ext byte */
				xfer8 (I_WRHL1, 0);
				xfer8 (I_WRHL2, 0);
				break;
			}
		}
		/* Wait for end of internal process */
		return (wait_ready ());

	case ISP:
		switch (dst) {
		case F_LOW:
			if (device->fuse_type <= 2) {	/* Type 1,2 */
				spi_xmit (ISP_C_WR_FLB);
				spi_xmit ((byte)(val & 0xBF));
				spi_xmit (0);
				spi_xmit (0);
			} else {			/* Type 3..6 */
				spi_xmit (ISP_C_WR_FLB);
				spi_xmit (ISP_C_WR_FLBL);
				spi_xmit (0);
				spi_xmit (val);
			}
			break;

		case F_HIGH:
			spi_xmit (ISP_C_WR_FLB);
			spi_xmit (ISP_C_WR_FLBH);
			spi_xmit (0);
			spi_xmit (val);
			break;

		case F_EXTEND:
			spi_xmit (ISP_C_WR_FLB);
			spi_xmit (ISP_C_WR_FLBX);
			spi_xmit (0);
			spi_xmit (val);
			break;

		case F_LOCK:
			if (device->fuse_type <= 3) {	/* Type 0..3 */
				spi_xmit (ISP_C_WR_FLB);
				spi_xmit ((byte)(ISP_C_WR_FLBK | val));
				spi_xmit (0);
				spi_xmit (0);
			} else {			/* Type 4..6 */
				spi_xmit (ISP_C_WR_FLB);
				spi_xmit (ISP_C_WR_FLBK);
				spi_xmit (0);
				spi_xmit (val);
			}
		} /* switch (dst) */
		delay_ms (20);	/* Wait for 20ms */
		return (0);	/* Ok */

	default:
		return (1);
	}
}


/****************************************************************************
* ࠭ ⠫.
* 頥:
*	0 -- ࠭ 訫 ᯥ譮.
*	1 -- 訡, . ⠩      ०
*		ணࠬ஢  ISP.
****************************************************************************/

int erase_memory (void) {

	switch (port_mode) {
	case PAR:
		set_byte (XA_1, C_ERASE);
		stb_wr (0, device->erase_wait);
		return (wait_ready ());

	case HVS:
	case HVS15:
		xfer8 (I_LDCMD, C_ERASE);
		xfer8 (I_WRLL1, 0);
		xfer8 (I_WRLL2, 0);
		delay_ms (device->erase_wait);
		/* EEPROM  tiny11/12/15/s2323/s2343  ⭮ */
		if (device->eeprom_page == 0)
			xfer8 (I_LDCMD, C_NOP);
		return (wait_ready ());

	case ISP:
		spi_xmit (ISP_C_ERASE1);
		spi_xmit (ISP_C_ERASE2);
		spi_xmit (0);
		spi_xmit (0);

		delay_ms (40);

		if (strcmp (device->name, "tiny12") == 0) {
			if (enter_ispmode ())
				return (1);
			write_byte (FLASH_NS, 0, 0xff);

			spi_xmit (ISP_C_ERASE1);
			spi_xmit (ISP_C_ERASE2);
			spi_xmit (0);
			spi_xmit (0);

			delay_ms (40);
		}
		return (enter_ispmode ());

	default:
		return (1);
	}
}


/** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* E0F
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/
