/** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* BiDiPro/AVR v0.1.0                                                  avr.h
* Copyright (c) Oleg O. Chukaev, 2007, 2008  <oleg.chukaev@mail.ru>
* Based on AVRPP - AVR Parallel Programming Controller  R0.39 (C)ChaN, 2007
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/


#ifndef AVR_AVR_H
#define AVR_AVR_H


typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;


/*  ᠭ ᢮ ப஫ */
typedef struct _DEVPROP {
	char	name[20];	/* Device name */
	byte	sign[3];	/* Device signature bytes */
	dword	flash_size;	/* Flash memory size in unit of byte */
	word	flash_page;	/* Flash page size (0 is byte-by-byte) */
	dword	eeprom_size;	/* EEPROM size in unit of byte */
	word	eeprom_page;	/* EEPROM page size (0 is byte-by-byte) */
	word	erase_wait;	/* Wait time for chip erase (0 is polling) */
	word	fuse_wait;	/* Wait time for fuse write (0 is poiling) */
	byte	lock_data;	/* Default lock byte (program LB1 and LB2) */
	char	fuses;		/* Number of fuses */
	char	cals;		/* Number of calibration bytes */
	byte	fuse_mask[3];	/* Valid fuse bit mask [low, high, ext] */
	byte	fuse_default[3];/* Fuse default value [low, high, ext] */
	int	fuse_type;	/*  Fuse-⮢ */
	int	flash_wait;	/* ISP: প ᫥  Flash */
	int	eeprom_wait;	/* ISP: প ᫥  EEPROM */
	int	poll_data;	/* ISP: poll data */
	int	socket;		/* Type of socket: 0..7 */
} DEVPROP;



/* Byte read/write identifier */
#define FLASH		0
#define EEPROM		1
#define SIGNATURE	2
#define	CALIBS		3
#define FUSE		4
#define FLASH_NS	5


/* Fuse read/write identifier */
#define F_LOW		0
#define	F_HIGH		1
#define	F_EXTEND	2
#define	F_LOCK		3


/* Read a byte from device */
byte read_byte (int src, dword adr);

/* Write a byte into memory */
int write_byte (int dst, dword adr, byte wd);

/* Write a page into memory */
int write_page (int dst, dword adr, const byte *wd);

/* Write Fuse or Lock byte */
int write_fuselock (int dst, byte val);

/* Chip erase */
int erase_memory (void);


#endif /* AVR_AVR_H */

/** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* E0F
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/

