/** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* BiDiPro/AVR v0.1.0                                                  hex.c
* Copyright (c) Oleg O. Chukaev, 2007, 2008  <oleg.chukaev@mail.ru>
* Based on AVRPP - AVR Parallel Programming Controller  R0.39 (C)ChaN, 2007
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/


/*-------------------------------------------------------------------------
  ᮪ 
---------------------------------------------------------------------------

2007-07-05	0leg	-
2007-09-22	0leg	hex_block_size ७ᥭ  hex.c

-------------------------------------------------------------------------*/


#define DEFAULT_HEX_BLOCK_SIZE 16


#include <stdio.h>


typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;


/*     뢮  hex */
int hex_block_size = DEFAULT_HEX_BLOCK_SIZE;


/****************************************************************************
* ⥭ ⭠ 祭  hex-.
* ࠬ:
*	lp    -- 㪠⥫  㪠⥫   ப.
*	conut -- ⢮ , ⠥  ப (2, 4, 6, 8).
*	sum   -- 㪠⥫  ஫ 㬬.
* 頥:
*	⠭ 祭.
****************************************************************************/

static dword get_valh (char **lp, int count, byte *sum) {

	dword val = 0;
	byte n;

	while (count--) {
		n = *(*lp)++;
		if ((n -= '0') >= 10) {
			if ((n -= 7) < 10)
				return (0xFFFFFFFFL);
			if (n > 0xF)
				return (0xFFFFFFFFL);
		}
		val = (val << 4) + n;
		if ((count & 1) == 0)
			*sum += (byte)val;
	}
	return (val);
}


/****************************************************************************
* ⥭ Intel/Motorola hex-䠩  .
* ࠬ:
*	fp       -- 㪠⥫  䠩,  ண ந ⥭.
*	buffer   -- 㪠⥫  .
*	buffsize -- ࠧ .
*	datasize -- ᪮쪮  ॠ쭮 ⠭.
* 頥:
*	0  -- ⨣  䠩
*	-1 -- 訡
*	>1 -- 訡  hex-, 頥  ப
****************************************************************************/

long input_hexfile (FILE *fp, byte *buffer, dword buffsize, dword *datasize) {

	char line[600];		/* line input buffer */
	char *lp;		/* line read pointer */
	long lnum = 0;		/* input line number */
	word seg = 0, hadr = 0;	/* address expantion values for intel hex */
	dword addr, count, n;
	byte sum;

	while (fgets (line, sizeof(line), fp) != NULL) {
		lnum++;
		lp = &line[1]; sum = 0;

		if (line[0] == ':') {	/* Intel Hex format */
			if ((count = get_valh (&lp, 2, &sum)) > 0xFF)
				return (lnum);	/* byte count */
			if ((addr = get_valh (&lp, 4, &sum)) > 0xFFFF)
				return (lnum);	/* offset */

			switch (get_valh (&lp, 2, &sum)) {	/* block type? */
			case 0x00 :	/* data block */
				addr += (seg << 4) + (hadr << 16);
				while (count--) {
					n = get_valh (&lp, 2, &sum);
					if (n > 0xFF)
						return (lnum);
					if (addr >= buffsize)
						continue;		/* clip by buffer size */
					buffer[addr++] = (byte)n;	/* store the data */
					if (addr > *datasize)		/* update data size information */
						*datasize = addr;
				}
				break;

			case 0x01 :	/* end */
				if (count != 0)
					return (lnum);
				break;

			case 0x02 :	/* segment base [19:4] */
				if (count != 2)
					return (lnum);
				seg = (word)get_valh (&lp, 4, &sum);
				if (seg == 0xFFFF)
					return (lnum);
				break;

			case 0x03 :	/* program start address (segment:offset) */
				if (count != 4)
					return (lnum);
				get_valh (&lp, 8, &sum);
				break;

			case 0x04 :	/* high address base [31:16] */
				if (count != 2)
					return (lnum);
				hadr = (word)get_valh (&lp, 4, &sum);
				if (hadr == 0xFFFF)
					return (lnum);
				break;

			case 0x05 :	/* program start address (linear) */
				if (count != 4)
					return (lnum);
				get_valh (&lp, 8, &sum);
				break;

			default:	/* invalid block */
				return (lnum);
			} /* switch */
			if (get_valh (&lp, 2, &sum) > 0xFF)
				return (lnum);	/* get check sum */
			if (sum)
				return (lnum);							/* test check sum */
		} /* if */

		if (line[0] == 'S') {	/* Motorola S format */
			if ((*lp >= '1') && (*lp <= '3')) {
				switch (*lp++) {	/* record type? (S1/S2/S3) */
				case '1' :
					count = get_valh (&lp, 2, &sum) - 3;
					if (count > 0xFF)
						return (lnum);
					addr = get_valh (&lp, 4, &sum);
					if (addr == 0xFFFFFFFFL)
						return (lnum);
					break;

				case '2' :
					count = get_valh (&lp, 2, &sum) - 4;
					if (count > 0xFF)
						return (lnum);
					addr = get_valh (&lp, 6, &sum);
					if (addr == 0xFFFFFFFFL)
						return (lnum);
					break;

				default :
					count = get_valh (&lp, 2, &sum) - 5;
					if (count > 0xFF)
						return (lnum);
					addr = get_valh (&lp, 8, &sum);
					if (addr == 0xFFFFFFFFL)
						return (lnum);
				} /* switch */
				while (count--) {
					n = get_valh (&lp, 2, &sum);
					if (n > 0xFF)
						return (lnum);
					if (addr >= buffsize)
						continue;	/* clip by buffer size */
					buffer[addr++] = (byte)n;		/* store the data */
					if (addr > *datasize)			/* update data size information */
						*datasize = addr;
				}
				if (get_valh (&lp, 2, &sum) > 0xFF)
					return (lnum);	/* get check sum */
				if (sum != 0xFF)
					return (lnum);					/* test check sum */
			} /* if */
		} /* if */
	} /* while (fgets... */

	return (feof (fp) ? 0 : -1);
}


/****************************************************************************
*  Intel hex-  䠩.
* ࠬ:
*	fp     -- 㪠⥫  䠩,   ந .
*	buffer -- 㪠⥫  .
*	ofs    -- ᬥ饭  .
*	count  -- ⢮  .
*	type   -- ⨯ .
****************************************************************************/

static int put_hexline (FILE *fp, const byte *buffer, word ofs, byte count, byte type) {

	byte sum;
	int rc;

	/* Byte count, Offset address and Record type */
	rc = fprintf (fp, ":%02X%04X%02X", count, ofs, type);
	if (rc == EOF)
		return (1);
	sum = count + (ofs >> 8) + ofs + type;

	/* Data bytes */
	while (count--) {
		rc = fprintf (fp, "%02X", *buffer);
		if (rc == EOF)
			return (1);
		sum += *buffer++;
	}

	/* Check sum */
	rc = fprintf (fp, "%02X\n", (byte)-sum);
	if (rc == EOF)
		return (1);

	return (0);
}


/****************************************************************************
*    䠩  ଠ Intel hex.
* ࠬ:
*	fp       -- 㪠⥫  䠩,   ந .
*	buffer   -- 㪠⥫  .
*	datasize -- ᪮쪮 ⮢ 뢠.
*	bs       -- ࠧ hex- (0, 1, 2, 4, ..., 128), 0 -- .
*		    祭  㬮砭 (hex_block_size).
****************************************************************************/

int output_hexfile (FILE *fp, const byte *buffer, dword datasize, int bs) {

	word seg = 0, ofs = 0;
	byte segbuff[2], d, n;
	dword bc = datasize;
	int blocksize = bs ? bs : hex_block_size;
	int rc;

	while (bc) {
		if ((ofs == 0) && (datasize > 0x10000L)) {
			segbuff[0] = (byte)(seg >> 8); segbuff[1] = (byte)seg;
			rc = put_hexline (fp, segbuff, 0, 2, 2);
			if (rc)
				return (1);
			seg += 0x1000;
		}
		if (bc >= blocksize) {		/* full data block */
			for (d = 0xFF, n = 0; n < blocksize; n++)
				d &= *(buffer+n);
			if (d != 0xFF) {
				rc = put_hexline (fp, buffer, ofs, blocksize, 0);
				if (rc)
					return (1);
			}
			buffer += blocksize;
			bc -= blocksize;
			ofs += blocksize;
		} else {			/* fractional data block */
			for (d = 0xFF, n = 0; n < bc; n++)
				d &= *(buffer+n);
			if (d != 0xFF) {
				rc = put_hexline (fp, buffer, ofs, (byte)bc, 0);
				if (rc)
					return (1);
			}
			bc = 0;
		}
	}
	rc = put_hexline (fp, NULL, 0, 0, 1);	/* End block */
	if (rc)
		return (1);

	return (0);
}


/** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* E0F
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **/

